@@ -1,26 +1,32 @@
-from __future__ import unicode_literals
-
+from django import VERSION as DJANGO_VERSION
 from django.core.files.base import File
 from django.db.models.fields.files import (
     FieldFile,
     ImageFieldFile,
     ImageFileDescriptor
 )
-from django.utils import six
 
 from .mixins import VersatileImageMixIn
 
 
 class VersatileImageFieldFile(VersatileImageMixIn, ImageFieldFile):
 
+    def __setstate__(self, state):
+        self.__dict__.update(state)
+        if DJANGO_VERSION[0] >= 3 and DJANGO_VERSION[1] > 0:
+            self.storage = self.field.storage
+            self._create_on_demand = state.get('_create_on_demand')
+            self._ppoi_value = state.get('_ppoi_value')
+
     def __getstate__(self):
         # VersatileImageFieldFile needs access to its associated model field
         # and an instance it's attached to in order to work properly, but the
         # only necessary data to be pickled is the file's name itself.
         # Everything else will be restored later, by
         # VersatileImageFileDescriptor below.
-        state = super(VersatileImageFieldFile, self).__getstate__()
+        state = super().__getstate__()
         state['_create_on_demand'] = self._create_on_demand
+        state['_ppoi_value'] = self._ppoi_value
         return state
 
 
@@ -28,7 +34,7 @@ class VersatileImageFileDescriptor(ImageFileDescriptor):
 
     def __set__(self, instance, value):
         previous_file = instance.__dict__.get(self.field.name)
-        super(VersatileImageFileDescriptor, self).__set__(instance, value)
+        super().__set__(instance, value)
 
         # Updating ppoi_field on attribute set
         if previous_file is not None:
@@ -36,10 +42,8 @@ class VersatileImageFileDescriptor(ImageFileDescriptor):
             self.field.update_ppoi_field(instance)
 
     def __get__(self, instance=None, owner=None):
-        if instance is None:
-            raise AttributeError(
-                "The '%s' attribute can only be accessed from %s instances."
-                % (self.field.name, owner.__name__))
+        if instance is None:  # pragma: no cover
+            return self
 
         # This is slightly complicated, so worth an explanation.
         # instance.file`needs to ultimately return some instance of `File`,
@@ -55,7 +59,7 @@ class VersatileImageFileDescriptor(ImageFileDescriptor):
         # in __set__.
         file = instance.__dict__[self.field.name]
 
-        # Call the placeholder procecess method on VersatileImageField.
+        # Call the placeholder process method on VersatileImageField.
         # (This was called inside the VersatileImageField __init__ before) Fixes #28
         self.field.process_placeholder_image()
 
@@ -66,7 +70,7 @@ class VersatileImageFileDescriptor(ImageFileDescriptor):
         # it's also conceivable that user subclasses might also want to
         # subclass the attribute class]. This object understands how to convert
         # a path to a file, and also how to handle None.
-        if isinstance(file, six.string_types) or file is None:
+        if isinstance(file, str) or file is None:
             attr = self.field.attr_class(
                 instance=instance,
                 field=self.field,
@@ -78,7 +82,6 @@ class VersatileImageFileDescriptor(ImageFileDescriptor):
                 ppoi = instance.__dict__[attr.field.ppoi_field]
                 # ...and assigning it to VersatileImageField instance
                 attr.ppoi = ppoi
-
             instance.__dict__[self.field.name] = attr
 
         # Other types of files may be assigned as well, but they need to have
@@ -94,7 +97,7 @@ class VersatileImageFileDescriptor(ImageFileDescriptor):
         # Finally, because of the (some would say boneheaded) way pickle works,
         # the underlying FieldFile might not actually itself have an associated
         # file. So we need to reset the details of the FieldFile in those cases
-        elif isinstance(file, FieldFile) and not hasattr(file, 'field'):
+        elif isinstance(file, FieldFile) and not hasattr(file, 'field'):  # pragma: no cover
             file.instance = instance
             file.field = self.field
             file.storage = self.field.storage
@@ -104,4 +107,8 @@ class VersatileImageFileDescriptor(ImageFileDescriptor):
                 file.ppoi = ppoi
 
         # That was fun, wasn't it?
+        # Finally, ensure all the sizers/filters are available after pickling
+        to_return = instance.__dict__[self.field.name]
+        to_return.build_filters_and_sizers(to_return.ppoi, to_return.create_on_demand)
+        instance.__dict__[self.field.name] = to_return
         return instance.__dict__[self.field.name]
