@@ -1,12 +1,9 @@
-from __future__ import unicode_literals
-
 from functools import reduce
 import logging
 from sys import stdout
 
 from django.db.models import Model
 from django.db.models.query import QuerySet
-from django.utils import six
 
 from .utils import (
     get_rendition_key_set,
@@ -87,7 +84,7 @@ class VersatileImageFieldWarmer(object):
                 "{}".format(self.__class__.__name__)
             )
         self.queryset = queryset
-        if isinstance(rendition_key_set, six.string_types):
+        if isinstance(rendition_key_set, str):
             rendition_key_set = get_rendition_key_set(rendition_key_set)
         self.size_key_list = [
             size_key
@@ -115,7 +112,7 @@ class VersatileImageFieldWarmer(object):
         versatileimagefieldfile.create_on_demand = True
         try:
             url = get_url_from_image_key(versatileimagefieldfile, size_key)
-        except:
+        except Exception:  # pragma: no cover
             success = False
             url_or_filepath = versatileimagefieldfile.name
             logger.exception('Thumbnail generation failed',
@@ -146,11 +143,12 @@ class VersatileImageFieldWarmer(object):
                     num_images_pre_warmed += 1
                     if self.verbose:
                         cli_progress_bar(num_images_pre_warmed, total)
-                else:
+                else:  # pragma: no cover
                     failed_to_create_image_path_list.append(url_or_filepath)
 
-                if a * b == total:
+                if a * b == total and self.verbose:
                     stdout.write('\n')
 
-        stdout.flush()
+        if self.verbose:
+            stdout.flush()
         return (num_images_pre_warmed, failed_to_create_image_path_list)
